#
#   fraction.rb - 
# --
#   Usage:
#   class Rational < Numeric
#      (include Compareable)
#
#   Rational(a, b) --> a/b
#
#   Rational::+
#   Rational::-
#   Rational::*
#   Rational::/
#   Rational::%
#   Rational::divmod
#   Rational::abs
#   Rational::<=>
#   Rational::to_i
#   Rational::to_f
#   Rational::to_s
#
#   Integer::to_r
#
#

def Rational(a, b = 1)
  if a.kind_of?(Rational) && b == 1
    a
  else
    Rational.new((Rational::UNIT * a) / b)
  end
end
  
class Rational < Numeric

  UNIT = 86400000000

  def initialize(num)
    @num = num
  end
  
  def + (a)
    if a.kind_of?(Rational)
      Rational.new(num + a.num)
    elsif a.kind_of?(Integer)
      Rational.new(num + a * UNIT)
    elsif a.kind_of?(Float)
      Float(self) + a
    else
      x , y = a.coerce(self)
      x + y
    end
  end
  
  def - (a)
    if a.kind_of?(Rational)
      Rational.new(num - a.num)
    elsif a.kind_of?(Integer)
      Rational.new(num - a * UNIT)
    elsif a.kind_of?(Float)
      Float(self) - a
    else
      x , y = a.coerce(self)
      x - y
    end
  end
  
  def * (a)
    if a.kind_of?(Rational)
      Rational.new(num * a.num / UNIT)
    elsif a.kind_of?(Integer)
      Rational.new(num * a)
    elsif a.kind_of?(Float)
      Float(self) * a
    else
      x , y = a.coerce(self)
      x * y
    end
  end
  
  def / (a)
    if a.kind_of?(Rational)
      raise ZeroDivisionError, "devided by 0" if a.num == 0
      Rational.new(num * UNIT / a.num)
    elsif a.kind_of?(Integer)
      raise ZeroDivisionError, "devided by 0" if a == 0
      Rational.new(num / a)
    elsif a.kind_of?(Float)
      Float(self) / a
    else
      x , y = a.coerce(self)
      x / y
    end
  end
  
  def % (other)
    value = (self / other).to_i
    return self - other * value
  end
  
  def divmod(other)
    value = (self / other).to_i
    return value, self - other * value
  end
  
  def abs
    if @num > 0
      Rational.new(@num)
    else
      Rational.new(-@num)
    end
  end

  def <=> (other)
    if other.kind_of?(Rational)
      return num <=> other.num
    elsif other.kind_of?(Integer)
      return self <=> Rational.new(other * UNIT)
    elsif other.kind_of?(Float)
      return Float(self) <=> other
    else
      x , y = other.coerce(self)
      return x <=> y
    end
  end

  def coerce(other)
    if other.kind_of?(Float)
      return other, self.to_f
    elsif other.kind_of?(Integer)
      return Rational.new(other * UNIT), self
    else
      super
    end
  end

  def to_i
    Integer(@num.div(UNIT))
  end
  
  def to_f
    @num.to_f/UNIT
  end
  
  def to_s
    to_f.to_s
  end
  
  def to_r
    self
  end
  
  def inspect
    sprintf("Rational(%s)", @num.inspect)
  end
  
  def hash
    (@num >> 31)
  end
  
  attr :num
  
  private :initialize
end

class Integer
  def to_r
    Rational.new(self * Rational::UNIT)
  end
end

class Fixnum
  alias div! /;
  def div(other)
    if other.kind_of?(Fixnum)
      self.div!(other)
    elsif other.kind_of?(Bignum)
      x, y = other.coerce(self)
      x.div!(y)
    else
      x, y = other.coerce(self)
      x / y
    end
  end
end
#  alias divmod! divmod
  
class Bignum
  alias div! /;
  alias div /;
  alias divmod! divmod
end

