# indiab.rb: written by Takashi SUGA 2002-03-20 v0.3(suchowan@box.email.ne.jp)
#
#  I N   N DDDD  I   A   BBBB 
#  I NN  N D   D I  A A  B   B
#  I N N N D   D I A   A BBBB 
#  I N  NN D   D I AAAAA B   B
#  I N   N DDDD  I A   A BBBB 
#
=begin #############################################################
#=IndiaB                                                           #
\===Ch                                                       #
\                                                                  #
\==X[p[NX:                                                 #
$==Super Class:                                                    #
#*EMD+                                                             #
=end ###############################################################

class IndiaB  < EMD

# White / black  month and leap month identification table

               #[w]/b  [n]/l      #1  #2  #3
  LEAP_MAP  = {[false, false] => [ 0,  0,  0],
               [false, true ] => [-3, -3, -3],
               [true,  false] => [+1, -1, -5],
               [true,  true ] => [-2, -4, -2]}

  LEAP_NAME = [[[ 1, "%s krisna paksa"      ],
                [ 0, "%s sukra paksa"       ],
                [-2, "adika %s krisna paksa"],
                [-3, "adika %s sukra paksa" ]],
               [[-1, "%s krisna paksa"      ],
                [ 0, "%s sukra paksa"       ],
                [-4, "adika %s krisna paksa"],
                [-3, "adika %s sukra paksa" ]],
               [[-5, "%s krisna paksa"      ],
                [ 0, "%s sukra paksa"       ],
                [-2, "adika %s krisna paksa"],
                [-3, "adika %s sukra paksa" ]]]

=begin #########################################################
\==NX \bh:                                            #
$==Class Method:                                               #
=end ###########################################################
  class << self

=begin ***********
#--- IndiaB.instance(key)
\  ɑΉCX^X𐶐܂B
\
\    key  CX^Xw肷镶
=end =============
    def instance(key)
      case key
      when /^IndiaB$/i
        IndiaB.new(
          'CycleOffset' =>  +23.25 / 30,
          'Ephemeris'   => '30L=>12S',
          'Timezone'    =>   0,
          'TimeUnits'   =>  [nil, nil, 60, 60, 60, 60],
          'IndiaBnStyle'=>  1,
          'MonthNames'  =>  %w(
            Caitra      Vaisakha  Jyaistha  Asadha
            Abhijit     Bhadra    Asvina    Kartika
            Agrahayana  Pausa     Magha     Phalguna
          ),
          'YearEpoch'   =>  78
        )
      else; nil
      end
    end
  end # of class method
=begin ***********
#--- IndiaB.new([quality={})
\  IndiaB+IuWFNg𐶐ĕԂ܂B
\
\    quality  @̐`nbV
=end =============
  def initialize(quality={})
    _copy_quality(quality)
    _qualify
  end

=begin #########################################################
\==veNebh CX^X \bh:                       #
$==Prptected Instance Method:                                  #
=end ###########################################################
  protected

=begin ***********
#--- IndiaB#_date_to_sdn(*date)
\  NEEnߒʓɕϊĕԂ܂B
\
\    date \z  N [, [,]]
=end =============
  def _date_to_sdn(*date)
    y, m, d, = date + [1, 1, 1]
    m = [m, 0] unless Array === m
    s  = 12 * (y+@YearEpoch)  + m[0] - 1 + @CycleOffset
    n  = d + 29 + @Ephemeris[0].cn_to_time(s).moddiv(30)[1] * 30
    n += 15 unless m[1] % 3 == 0
    6.times { |i|  
      dd = _sdn_to_date(n-=30)
      if m == dd[1]
        return n 
      end
    }
    return nil
  end

=begin ***********
#--- IndiaB#_sdn_to_date(sdn)
\  nߒʓNEEɕϊĕԂ܂B
\
\    sdn nߒʓ
=end =============
  def _sdn_to_date(sdn)
    t, m = sdn.moddiv(30)
    s    = (0..2).to_a.collect {|i| 
      (@Ephemeris[0].time_to_cn(30*(m+i)) - @CycleOffset).floor + 1
    }

    y = s[0].moddiv(12)[1]
    d = t.floor + 1

    b, n, f = false, false, s[0]==s[1]
    if (t>=15.0)
      d -=  15
      b  = true
      n  = true           if (@IndiaBnStyle >  1)
      f  = (s[1] == s[2]) if (@IndiaBnStyle == 2)
    end

    m = [(n ? s[1] : s[0] ).moddiv(12)[0] + 1,
          LEAP_MAP[[b,f]][@IndiaBnStyle-1]
        ]

    return y-@YearEpoch, m, d
  end

=begin ***********
#--- IndiaB#_qualify
\  IndiaB+IuWFNg̐`ݒ肵܂B
\
\  @Ephemeris[0]  = ʑ̌vZɗp錎 Ephemeris
\  @Timezone[0]   = ʑ̌vZɗp錎 Timezone
\  @CycleOffset   = ʑ̃ItZbg
\  @IndiaBnStyle  = [̕\@
\  @YearEpoch     = Ň
=end =============
  def _qualify

    # ʑ̃ItZbg(6ȈtɌ_܂ł̍΍)
    @CycleOffset  ||= +23.25 / 30
    
    @Ephemeris ||= '30L=>12S'
    unless (@Ephemeris = Ephemeris.get(@Ephemeris))
      @Ephemeris = Ephemeris.get('30L=>12S')
    end

    # g^C][
    @Timezone     ||=  [0]

    @TimeUnits    ||= [nil, nil, 60, 60, 60, 60]

    # [єE̕\@
    @IndiaBnStyle ||= 1
    @LeapNames    ||= LEAP_NAME[@IndiaBnStyle-1]

    # ̖O
    @MonthNames   ||= %w(
      Caitra      Vaisakha  Jyaistha  Asadha
      Abhijit     Bhadra    Asvina    Kartika
      Agrahayana  Pausa     Magha     Phalguna
    )

    # Ň(TJ)
    @YearEpoch    ||=   78
    super
  end

################################################################
  func :_date_to_sdn, :_sdn_to_date
end

